<?php

namespace App\Http\Controllers;

use App\Models\Cash;
use App\Models\Collectors;
use App\Models\Donators;
use App\Models\Receipt;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;


class ReceiptController extends Controller
{
   
    public function index(Request $request)
    {

        $collectorsdata = Collectors::all();

        $data_query = Receipt::select('*');
      
        if($request->input('from_date')){

            $from_date = $request->input('from_date');
            $to_date = $request->input('to_date');
        
            $data_query = $data_query->whereBetween('entry_date',[$from_date,$to_date]);
        }
        // else{

        //     $current_date = date('Y-m-d');
        //     $data_query = $data_query->where('entry_date',$current_date);
        // }

        if($request->input('receiver_id')){

            $data_query->where('collectors_id',$request->input('receiver_id'));
        }

        if($request->input('donator_id')){

            $data_query->where('donators_id',$request->input('donator_id'));
        }
       
        $data = $data_query->get();
            

        if($request->input('filter') == 'filter'){
            return view('receipt.table-list',['data' => $data]);
        }else{
            return view('receipt.list',['data' => $data,'collectorsdata'=>$collectorsdata]);
        }
    }

  
    public function create()
    {
        $collectorsdata = Collectors::all();

        return view('receipt.create',['collectorsdata'=>$collectorsdata]); 
    }

    
    public function store(Request $request)
    {
        $prefix = "REC";
        $sequence_padding = $this->generate_sequence();
        $receipt_no =  $prefix."".$sequence_padding;    
        
        $receiptEntry = new Receipt();
        
        $receiptEntry->entry_date = Carbon::parse($request->input('entry_date'))->format('Y-m-d');
        $receiptEntry->prefix = $prefix;
        $receiptEntry->sequence = $this->max_no;
        $receiptEntry->receipt_no = $receipt_no; 
        $receiptEntry->collectors_id = $request->input('collector_name');
        $receiptEntry->donators_id = $request->input('donators_id');
        $receiptEntry->amount = $request->input('amount');
        $receiptEntry->description = $request->input('notes');
        $receiptEntry->created_by = Auth::user()->id;
        $receiptEntry->save();

        $cashTransaction = new Cash();
        $cashTransaction->entry_date = Carbon::parse($request->input('entry_date'))->format('Y-m-d');
        $cashTransaction->refer = "Receipt";
        $cashTransaction->refer_id = $receiptEntry->id;
        $cashTransaction->collectors_id = $request->input('collector_name');
        $cashTransaction->debit = $request->input('amount');
        $cashTransaction->created_by = Auth::user()->id;
        $cashTransaction->save();

         return redirect('/receipt')->with('success','Receipt Entry Created For '.$receipt_no);
    }

    public function edit($id)
    {

        $data = Receipt::find($id);

        $collectorsdata = Collectors::all();

             $donatorsdata = Donators::where('receiver_name',$data->collector_name)->get();

            //$users_divisions = $userdata->division_id;

            return view('receipt.edit',['data' => $data,'donatorsdata'=>$donatorsdata,'collectorsdata'=>$collectorsdata]); 
    }

    public function update(Request $request, $id)
    {
        $receiptEntry = Receipt::find($id);
        
        $receiptEntry->entry_date = Carbon::parse($request->input('entry_date'))->format('Y-m-d');
        $receiptEntry->collectors_id = $request->input('collector_name');
        $receiptEntry->donators_id = $request->input('donators_id');
        $receiptEntry->amount = $request->input('amount');
        $receiptEntry->description = $request->input('notes');
        $receiptEntry->updated_by = Auth::user()->id;
        $receiptEntry->save();

        $cashTransactionDel = Cash::where('refer_id',$id)->where('refer','Receipt')->delete();

        $cashTransaction = new Cash();
        $cashTransaction->entry_date = Carbon::parse($request->input('entry_date'))->format('Y-m-d');
        $cashTransaction->refer = "Receipt";
        $cashTransaction->refer_id = $receiptEntry->id;
        $cashTransaction->collectors_id = $request->input('collector_name');
        $cashTransaction->debit = $request->input('amount');
        $cashTransaction->created_by = Auth::user()->id;
        $cashTransaction->save();

         return redirect('/receipt')->with('success','Receipt Entry Updated For '.$receiptEntry->receipt_no);
    }

   
    public function destroy($id)
    {
        $receiptEntry = Receipt::find($id);

        $cashTransactionDel = Cash::where('refer_id',$id)->where('refer','Receipt')->delete();

        $receiptEntry->delete();

    }

    public function generate_sequence(){
        
        $receiptEntry = new Receipt();
        $count = $receiptEntry->count('id');
        
        if($count == 0){
            
            $max_seq = 1;        
        }
        else{
            
            $max_seq = $receiptEntry->max('sequence') + 1;  
        }

        $this->max_no = $max_seq;
        $seq_padding = Str::of($max_seq)->padLeft(5,'0');

        return $seq_padding;
    }
}
